package org.processmining.acceptingpetrinetreductionlayouter.dialogs;

import java.awt.Dimension;

import javax.swing.DefaultListModel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.acceptingpetrinetreductionlayouter.parameters.LayoutParameters;
import org.processmining.acceptingpetrinetreductionlayouter.parameters.NamedColor;
import org.processmining.acceptingpetrinetreductionlayouter.parameters.ShowHideParameter;
import org.processmining.framework.util.ui.widgets.ProMList;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

public class LayoutDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5163579084060653646L;

	private LayoutParameters parameters;
	
	public LayoutDialog(LayoutParameters parameters) {
		this.parameters = parameters;
	}
	
	public JPanel getPanel(int n) {
		double size[][] = { { TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.FILL, 30, 30 } };
		setLayout(new TableLayout(size));
		
		DefaultListModel<ShowHideParameter> labelListModel = new DefaultListModel<ShowHideParameter>();
		for (ShowHideParameter value : ShowHideParameter.values()) {
			labelListModel.addElement(value);
		}
		final ProMList<ShowHideParameter> labelList = new ProMList<ShowHideParameter>("Select what to do with place labels",
				labelListModel);
		labelList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		labelList.setPreferredSize(new Dimension(100, 100));
		labelList.setSelection(parameters.getShowsPlaceLabels());
		labelList.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				// TODO Auto-generated method stub
				parameters.setShowPlaceLabels(labelList.getSelectedValuesList().get(0));
			}
			
		});
		add(labelList, "0, 0");
		
		DefaultListModel<NamedColor> colorListModel = new DefaultListModel<NamedColor>();
		for (NamedColor namedColor : NamedColor.values()) {
			colorListModel.addElement(namedColor);
		}
		final ProMList<NamedColor> colorList = new ProMList<NamedColor>("Select color for problematic arcs",
				colorListModel);
		colorList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		colorList.setPreferredSize(new Dimension(100, 100));
		colorList.setSelection(parameters.getNamedColor());
		colorList.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				// TODO Auto-generated method stub
				parameters.setNamedColor(colorList.getSelectedValuesList().get(0));
			}
			
		});
		add(colorList, "1, 0");
		
		final NiceSlider horizontalSpaceSlider = SlickerFactory.instance().createNiceIntegerSlider("Horizontal space", 1,
				100, parameters.getHorizontalSpace(), Orientation.HORIZONTAL);
		horizontalSpaceSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setHorizontalSpace(horizontalSpaceSlider.getSlider().getValue());
			}
		});
		add(horizontalSpaceSlider, "0, 1, 1, 1");

		final NiceSlider verticalSpaceSlider = SlickerFactory.instance().createNiceIntegerSlider("Vertical space", 1,
				100, parameters.getVerticalSpace(), Orientation.HORIZONTAL);
		verticalSpaceSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setVerticalSpace(verticalSpaceSlider.getSlider().getValue());
			}
		});
		add(verticalSpaceSlider, "0, 2, 1, 2");

		return this;
	}

}
