package org.processmining.acceptingpetrinetreductionlayouter.workspaces;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinetreductionlayouter.models.Block;
import org.processmining.acceptingpetrinetreductionlayouter.models.NodeBlock;
import org.processmining.acceptingpetrinetreductionlayouter.parameters.LayoutParameters;
import org.processmining.acceptingpetrinetreductionlayouter.utils.EdgeComparator;
import org.processmining.acceptingpetrinetreductionlayouter.utils.NodeComparator;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;

public class ReductionWorkspace {

	public final AcceptingPetriNet net;
	public final LayoutParameters parameters;
	public final AcceptingPetriNet workNet;
	public Map<Place, Place> placeMap;
	//	public Map<Transition, Transition> transitionMap;
	public Map<Transition, Place> transitionMap;
	public Map<Place, PetrinetNode> reversePlaceMap;
	public Map<Transition, Arc> reverseTransitionMap;

	public int nofPlaces;
	public int nofTransitions;

	//	public Map<Place, Integer> pMap;
	public Map<Transition, Integer> tMap;
	//	public Map<Pair<Place, Place>, Integer> mMap;

	public int[][] incidenceMatrix;

	public Map<PetrinetNode, List<PetrinetNode>> presets;
	public Map<PetrinetNode, List<PetrinetNode>> postsets;
	//	public List<Place> places;
	public List<Transition> transitions;
	//	public List<Arc> arcs;
	public List<Arc> ptArcs;

	public Map<PetrinetNode, Block> layoutMap;

	public Map<PetrinetNode, Pair<Double, Double>> layout;
	public double threshold;

	public GraphLayoutConnection layoutConnection;
	public final double SCALE = 1000.0;
	public final double OFFSET = 0.0;

	public Set<Transition> hardTransitions;

	public ReductionWorkspace(AcceptingPetriNet net) {
		this(net, new LayoutParameters());
	}

	public ReductionWorkspace(AcceptingPetriNet net, LayoutParameters parameters) {
		this.parameters = parameters;
		this.net = cloneNet(net);
		workNet = cloneWorkNet(this.net);
		update();
		layoutMap = new HashMap<PetrinetNode, Block>();
		for (PetrinetNode node : workNet.getNet().getNodes()) {
			layoutMap.put(node, new NodeBlock(node, this));
		}
		hardTransitions = new HashSet<Transition>();
	}

	private AcceptingPetriNet cloneNet(AcceptingPetriNet net) {
		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Petrinet reducedPN = cloneNet(net.getNet(), placeMap, transitionMap);
		Marking initialMarking = apply(net.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = apply(net.getFinalMarkings(), placeMap);
		return AcceptingPetriNetFactory.createAcceptingPetriNet(reducedPN, initialMarking, finalMarkings);
	}

	private AcceptingPetriNet cloneWorkNet(AcceptingPetriNet net) {
		placeMap = new HashMap<Place, Place>();
		//		transitionMap = new HashMap<Transition, Transition>();
		transitionMap = new HashMap<Transition, Place>();
		reversePlaceMap = new HashMap<Place, PetrinetNode>();
		reverseTransitionMap = new HashMap<Transition, Arc>();
		Petrinet reducedPN = cloneWorkNet(net.getNet());
		Marking initialMarking = apply(net.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = apply(net.getFinalMarkings(), placeMap);
		return AcceptingPetriNetFactory.createAcceptingPetriNet(reducedPN, initialMarking, finalMarkings);
	}

	private Petrinet cloneNet(Petrinet net, Map<Place, Place> placeMap, Map<Transition, Transition> transitionMap) {
		Petrinet clonedNet = PetrinetFactory.newPetrinet(net.getLabel());
		for (Place place : net.getPlaces()) {
			Place clonedPlace = clonedNet.addPlace(place.getLabel());
			placeMap.put(place, clonedPlace);
			for (String key : place.getAttributeMap().keySet()) {
				clonedPlace.getAttributeMap().put(key, place.getAttributeMap().get(key));
			}
			switch (parameters.getShowsPlaceLabels()) {
				case SHOW :
					clonedPlace.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
					break;
				case HIDE :
					clonedPlace.getAttributeMap().put(AttributeMap.SHOWLABEL, false);
					break;
				default :
					// No action required.
			}
		}
		for (Transition transition : net.getTransitions()) {
			Transition clonedTransition = clonedNet.addTransition(transition.getLabel());
			clonedTransition.setInvisible(transition.isInvisible());
			transitionMap.put(transition, clonedTransition);
			for (String key : transition.getAttributeMap().keySet()) {
				clonedTransition.getAttributeMap().put(key, transition.getAttributeMap().get(key));
			}
		}
		for (PetrinetEdge<?, ?> edge : net.getEdges()) {
			if (edge instanceof Arc) {
				Arc arc = (Arc) edge;
				Arc clonedArc;
				if (placeMap.containsKey(arc.getSource())) {
					clonedArc = clonedNet.addArc(placeMap.get(arc.getSource()), transitionMap.get(arc.getTarget()),
							arc.getWeight());
				} else {
					clonedArc = clonedNet.addArc(transitionMap.get(arc.getSource()), placeMap.get(arc.getTarget()),
							arc.getWeight());
				}
				for (String key : arc.getAttributeMap().keySet()) {
					clonedArc.getAttributeMap().put(key, arc.getAttributeMap().get(key));
				}
			}
		}
		return clonedNet;
	}

	private Petrinet cloneWorkNet(Petrinet net) {
		Petrinet clonedNet = PetrinetFactory.newPetrinet(net.getLabel());
		for (Place place : net.getPlaces()) {
			Place clonedPlace = clonedNet.addPlace(place.getLabel());
			placeMap.put(place, clonedPlace);
			reversePlaceMap.put(clonedPlace, place);
		}
		for (Transition transition : net.getTransitions()) {
			Place clonedPlace = clonedNet.addPlace(transition.getLabel());
			transitionMap.put(transition, clonedPlace);
			reversePlaceMap.put(clonedPlace, transition);
		}
		for (PetrinetEdge<?, ?> edge : net.getEdges()) {
			if (edge instanceof Arc) {
				Arc arc = (Arc) edge;
				Transition transition = clonedNet.addTransition(arc.getSource().getLabel() + " -> "
						+ arc.getTarget().getLabel());
				if (placeMap.containsKey(arc.getSource())) {
					clonedNet.addArc(placeMap.get(arc.getSource()), transition, arc.getWeight());
					clonedNet.addArc(transition, transitionMap.get(arc.getTarget()), arc.getWeight());
				} else {
					clonedNet.addArc(transitionMap.get(arc.getSource()), transition, arc.getWeight());
					clonedNet.addArc(transition, placeMap.get(arc.getTarget()), arc.getWeight());
				}
				reverseTransitionMap.put(transition, arc);
			}
		}
		return clonedNet;
	}

	private Marking apply(Marking marking, Map<Place, Place> map) {
		Marking appliedMarking = new Marking();
		for (Place place : marking.baseSet()) {
			appliedMarking.add(map.get(place), marking.occurrences(place));
		}
		return appliedMarking;
	}

	private Set<Marking> apply(Set<Marking> markings, Map<Place, Place> map) {
		Set<Marking> appliedMarkings = new HashSet<Marking>();
		for (Marking marking : markings) {
			appliedMarkings.add(apply(marking, map));
		}
		return appliedMarkings;
	}

	public void update() {

		presets = new HashMap<PetrinetNode, List<PetrinetNode>>();
		postsets = new HashMap<PetrinetNode, List<PetrinetNode>>();
		//		List<Place> places = new ArrayList<Place>();
		transitions = new ArrayList<Transition>();
		List<Arc> arcs = new ArrayList<Arc>();
		ptArcs = new ArrayList<Arc>();

		NodeComparator nodeComparator = new NodeComparator();
		EdgeComparator edgeComparator = new EdgeComparator();

		List<PetrinetNode> nodes = new ArrayList<PetrinetNode>(workNet.getNet().getNodes());
		Collections.sort(nodes, nodeComparator);
		for (PetrinetNode node : nodes) {
			presets.put(node, new ArrayList<PetrinetNode>());
			postsets.put(node, new ArrayList<PetrinetNode>());
		}

		List<PetrinetEdge<?, ?>> edges = new ArrayList<PetrinetEdge<?, ?>>(workNet.getNet().getEdges());
		Collections.sort(edges, edgeComparator);
		for (PetrinetEdge<?, ?> edge : edges) {
			if (edge instanceof Arc) {
				Arc arc = (Arc) edge;
				arcs.add(arc);
				if (arc.getSource() instanceof Place) {
					ptArcs.add(arc);
				}
				presets.get(arc.getTarget()).add(arc.getSource());
				postsets.get(arc.getSource()).add(arc.getTarget());
			}
		}

		for (PetrinetNode node : nodes) {
			Collections.sort(presets.get(node), nodeComparator);
			Collections.sort(postsets.get(node), nodeComparator);
		}

		Map<Place, Integer> pMap = new HashMap<Place, Integer>();
		nofPlaces = 0;
		for (Place place : workNet.getNet().getPlaces()) {
			//			places.add(place);
			pMap.put(place, nofPlaces);
			nofPlaces++;
		}
		tMap = new HashMap<Transition, Integer>();
		nofTransitions = 0;
		for (Transition transition : workNet.getNet().getTransitions()) {
			transitions.add(transition);
			tMap.put(transition, nofTransitions);
			nofTransitions++;
		}
		Set<Place> initialPlaces = new HashSet<Place>(workNet.getInitialMarking().baseSet());
		Set<Place> finalPlaces = new HashSet<Place>();
		for (Marking finalMarking : workNet.getFinalMarkings()) {
			finalPlaces.addAll(finalMarking.baseSet());
		}
		Map<Pair<Place, Place>, Integer> mMap = new HashMap<Pair<Place, Place>, Integer>();
		for (Place finalPlace : finalPlaces) {
			if (pMap.containsKey(finalPlace)) {
				for (Place initialPlace : initialPlaces) {
					if (pMap.containsKey(initialPlace)) {
						mMap.put(new Pair<Place, Place>(finalPlace, initialPlace), nofTransitions);
						nofTransitions++;
					}
				}
			}
		}

		incidenceMatrix = new int[nofPlaces][nofTransitions];

		for (int p = 0; p < nofPlaces; p++) {
			for (int t = 0; t < nofTransitions; t++) {
				incidenceMatrix[p][t] = 0;
			}
		}

		for (Arc arc : arcs) {
			if (arc.getSource() instanceof Place) {
				incidenceMatrix[pMap.get(arc.getSource())][tMap.get(arc.getTarget())] -= arc.getWeight();
			} else {
				incidenceMatrix[pMap.get(arc.getTarget())][tMap.get(arc.getSource())] += arc.getWeight();
			}
		}

		for (Place finalPlace : finalPlaces) {
			if (pMap.containsKey(finalPlace)) {
				for (Place initialPlace : initialPlaces) {
					if (pMap.containsKey(initialPlace)) {
						Pair<Place, Place> pair = new Pair<Place, Place>(finalPlace, initialPlace);
						incidenceMatrix[pMap.get(finalPlace)][mMap.get(pair)] -= 1;
						incidenceMatrix[pMap.get(initialPlace)][mMap.get(pair)] += 1;
					}
				}
			}
		}

		//		Collections.sort(places, nodeComparator);
		Collections.sort(transitions, nodeComparator);
		//		Collections.sort(arcs, edgeComparator);
		Collections.sort(ptArcs, edgeComparator);
	}
}
